@@ -2,6 +2,7 @@
 
 #include "rc_runtime_types.h"
 
+#include "../src/rapi/rc_api_common.h"
 #include "../test_framework.h"
 
 #define DOREQUEST_URL "https://retroachievements.org/dorequest.php"
@@ -18,6 +19,7 @@ static void test_init_resolve_hash_request() {
   ASSERT_NUM_EQUALS(rc_api_init_resolve_hash_request(&request, &resolve_hash_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=gameid&m=ABCDEF0123456789");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -32,6 +34,7 @@ static void test_init_resolve_hash_request_no_credentials() {
   ASSERT_NUM_EQUALS(rc_api_init_resolve_hash_request(&request, &resolve_hash_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=gameid&m=ABCDEF0123456789");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -99,6 +102,7 @@ static void test_init_fetch_game_data_request() {
   ASSERT_NUM_EQUALS(rc_api_init_fetch_game_data_request(&request, &fetch_game_data_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=patch&u=Username&t=API_TOKEN&g=1234");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -246,6 +250,140 @@ static void test_process_fetch_game_data_response_achievements() {
   rc_api_destroy_fetch_game_data_response(&fetch_game_data_response);
 }
 
+static void test_process_fetch_game_data_response_achievement_types() {
+  rc_api_fetch_game_data_response_t fetch_game_data_response;
+  const char* server_response = "{\"Success\":true,\"PatchData\":{"
+    "\"ID\":20,\"Title\":\"Another Amazing Game\",\"ConsoleID\":19,\"ImageIcon\":\"/Images/112233.png\","
+    "\"Achievements\":["
+    "{\"ID\":5501,\"Title\":\"Ach1\",\"Description\":\"Desc1\",\"Flags\":3,\"Points\":5,"
+     "\"MemAddr\":\"0=1\",\"Author\":\"User1\",\"BadgeName\":\"00234\",\"Type\":\"\","
+     "\"Created\":1367266583,\"Modified\":1376929305},"
+    "{\"ID\":5502,\"Title\":\"Ach2\",\"Description\":\"Desc2\",\"Flags\":3,\"Points\":2,"
+     "\"MemAddr\":\"0=2\",\"Author\":\"User1\",\"BadgeName\":\"00235\",\"Type\":\"missable\","
+     "\"Created\":1376970283,\"Modified\":1376970283},"
+    "{\"ID\":5503,\"Title\":\"Ach3\",\"Description\":\"Desc3\",\"Flags\":5,\"Points\":0,"
+     "\"MemAddr\":\"0=3\",\"Author\":\"User2\",\"BadgeName\":\"00236\",\"Type\":\"progression\","
+     "\"Created\":1376969412,\"Modified\":1376969412},"
+    "{\"ID\":5504,\"Title\":\"Ach4\",\"Description\":\"Desc4\",\"Flags\":3,\"Points\":10,"
+     "\"MemAddr\":\"0=4\",\"Author\":\"User1\",\"BadgeName\":\"00236\",\"Type\":\"win_condition\","
+     "\"Created\":1504474554,\"Modified\":1504474554},"
+    "{\"ID\":5505,\"Title\":\"Ach5 [m]\",\"Description\":\"Desc5\",\"Flags\":3,\"Points\":10,"
+     "\"MemAddr\":\"0=4\",\"Author\":\"User1\",\"BadgeName\":\"00236\",\"Type\":\"\","
+     "\"Created\":1504474554,\"Modified\":1504474554},"
+    "{\"ID\":5506,\"Title\":\"[m] Ach6\",\"Description\":\"Desc6\",\"Flags\":3,\"Points\":10,"
+     "\"MemAddr\":\"0=4\",\"Author\":\"User1\",\"BadgeName\":\"00236\",\"Type\":\"\","
+     "\"Created\":1504474554,\"Modified\":1504474554},"
+    "{\"ID\":5507,\"Title\":\"Ach7\",\"Description\":\"Desc7\",\"Flags\":3,\"Points\":5,"
+     "\"MemAddr\":\"0=1\",\"Author\":\"User1\",\"BadgeName\":\"00234\","
+     "\"Created\":1367266583,\"Modified\":1376929305}"
+    "],\"Leaderboards\":[]"
+    "}}";
+  rc_api_achievement_definition_t* achievement;
+
+  memset(&fetch_game_data_response, 0, sizeof(fetch_game_data_response));
+
+  ASSERT_NUM_EQUALS(rc_api_process_fetch_game_data_response(&fetch_game_data_response, server_response), RC_OK);
+  ASSERT_NUM_EQUALS(fetch_game_data_response.response.succeeded, 1);
+  ASSERT_PTR_NULL(fetch_game_data_response.response.error_message);
+  ASSERT_NUM_EQUALS(fetch_game_data_response.num_achievements, 7);
+
+  ASSERT_PTR_NOT_NULL(fetch_game_data_response.achievements);
+  achievement = fetch_game_data_response.achievements;
+
+  ASSERT_NUM_EQUALS(achievement->id, 5501);
+  ASSERT_STR_EQUALS(achievement->title, "Ach1");
+  ASSERT_NUM_EQUALS(achievement->type, RC_ACHIEVEMENT_TYPE_STANDARD);
+
+  ++achievement;
+  ASSERT_NUM_EQUALS(achievement->id, 5502);
+  ASSERT_STR_EQUALS(achievement->title, "Ach2");
+  ASSERT_NUM_EQUALS(achievement->type, RC_ACHIEVEMENT_TYPE_MISSABLE);
+
+  ++achievement;
+  ASSERT_NUM_EQUALS(achievement->id, 5503);
+  ASSERT_STR_EQUALS(achievement->title, "Ach3");
+  ASSERT_NUM_EQUALS(achievement->type, RC_ACHIEVEMENT_TYPE_PROGRESSION);
+
+  ++achievement;
+  ASSERT_NUM_EQUALS(achievement->id, 5504);
+  ASSERT_STR_EQUALS(achievement->title, "Ach4");
+  ASSERT_NUM_EQUALS(achievement->type, RC_ACHIEVEMENT_TYPE_WIN);
+
+  ++achievement;
+  ASSERT_NUM_EQUALS(achievement->id, 5505);
+  ASSERT_STR_EQUALS(achievement->title, "Ach5");
+  ASSERT_NUM_EQUALS(achievement->type, RC_ACHIEVEMENT_TYPE_MISSABLE);
+
+  ++achievement;
+  ASSERT_NUM_EQUALS(achievement->id, 5506);
+  ASSERT_STR_EQUALS(achievement->title, "Ach6");
+  ASSERT_NUM_EQUALS(achievement->type, RC_ACHIEVEMENT_TYPE_MISSABLE);
+
+  ++achievement;
+  ASSERT_NUM_EQUALS(achievement->id, 5507);
+  ASSERT_STR_EQUALS(achievement->title, "Ach7");
+  ASSERT_NUM_EQUALS(achievement->type, RC_ACHIEVEMENT_TYPE_STANDARD);
+
+  rc_api_destroy_fetch_game_data_response(&fetch_game_data_response);
+}
+
+static void test_process_fetch_game_data_response_achievement_rarity() {
+  rc_api_fetch_game_data_response_t fetch_game_data_response;
+  const char* server_response = "{\"Success\":true,\"PatchData\":{"
+    "\"ID\":20,\"Title\":\"Another Amazing Game\",\"ConsoleID\":19,\"ImageIcon\":\"/Images/112233.png\","
+    "\"Achievements\":["
+    "{\"ID\":5501,\"Title\":\"Ach1\",\"Description\":\"Desc1\",\"Flags\":3,\"Points\":5,"
+     "\"MemAddr\":\"0=1\",\"Author\":\"User1\",\"BadgeName\":\"00234\",\"Rarity\":100.0,\"RarityHardcore\":66.67,"
+     "\"Created\":1367266583,\"Modified\":1376929305},"
+    "{\"ID\":5502,\"Title\":\"Ach2\",\"Description\":\"Desc2\",\"Flags\":3,\"Points\":2,"
+     "\"MemAddr\":\"0=2\",\"Author\":\"User1\",\"BadgeName\":\"00235\",\"Rarity\":57.43,\"RarityHardcore\":57.43,"
+     "\"Created\":1376970283,\"Modified\":1376970283},"
+    "{\"ID\":5503,\"Title\":\"Ach3\",\"Description\":\"Desc3\",\"Flags\":5,\"Points\":0,"
+     "\"MemAddr\":\"0=3\",\"Author\":\"User2\",\"BadgeName\":\"00236\",\"Rarity\":6.8,\"RarityHardcore\":0,"
+     "\"Created\":1376969412,\"Modified\":1376969412},"
+    "{\"ID\":5504,\"Title\":\"Ach4\",\"Description\":\"Desc4\",\"Flags\":5,\"Points\":0,"
+     "\"MemAddr\":\"0=3\",\"Author\":\"User2\",\"BadgeName\":\"00236\","
+     "\"Created\":1376969412,\"Modified\":1376969412}"
+    "],\"Leaderboards\":[]"
+    "}}";
+  rc_api_achievement_definition_t* achievement;
+
+  memset(&fetch_game_data_response, 0, sizeof(fetch_game_data_response));
+
+  ASSERT_NUM_EQUALS(rc_api_process_fetch_game_data_response(&fetch_game_data_response, server_response), RC_OK);
+  ASSERT_NUM_EQUALS(fetch_game_data_response.response.succeeded, 1);
+  ASSERT_PTR_NULL(fetch_game_data_response.response.error_message);
+  ASSERT_NUM_EQUALS(fetch_game_data_response.num_achievements, 4);
+
+  ASSERT_PTR_NOT_NULL(fetch_game_data_response.achievements);
+  achievement = fetch_game_data_response.achievements;
+
+  ASSERT_NUM_EQUALS(achievement->id, 5501);
+  ASSERT_STR_EQUALS(achievement->title, "Ach1");
+  ASSERT_FLOAT_EQUALS(achievement->rarity, 100.0f);
+  ASSERT_FLOAT_EQUALS(achievement->rarity_hardcore, 66.67f);
+
+  ++achievement;
+  ASSERT_NUM_EQUALS(achievement->id, 5502);
+  ASSERT_STR_EQUALS(achievement->title, "Ach2");
+  ASSERT_FLOAT_EQUALS(achievement->rarity, 57.43f);
+  ASSERT_FLOAT_EQUALS(achievement->rarity_hardcore, 57.43f);
+
+  ++achievement;
+  ASSERT_NUM_EQUALS(achievement->id, 5503);
+  ASSERT_STR_EQUALS(achievement->title, "Ach3");
+  ASSERT_FLOAT_EQUALS(achievement->rarity, 6.8f);
+  ASSERT_FLOAT_EQUALS(achievement->rarity_hardcore, 0.0f);
+
+  ++achievement;
+  ASSERT_NUM_EQUALS(achievement->id, 5504);
+  ASSERT_STR_EQUALS(achievement->title, "Ach4");
+  ASSERT_FLOAT_EQUALS(achievement->rarity, 100.0f);
+  ASSERT_FLOAT_EQUALS(achievement->rarity_hardcore, 100.0f);
+
+  rc_api_destroy_fetch_game_data_response(&fetch_game_data_response);
+}
+
 static void test_process_fetch_game_data_response_leaderboards() {
   rc_api_fetch_game_data_response_t fetch_game_data_response;
   const char* server_response = "{\"Success\":true,\"PatchData\":{"
@@ -385,6 +523,7 @@ static void test_init_ping_request() {
   ASSERT_NUM_EQUALS(rc_api_init_ping_request(&request, &ping_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=ping&u=Username&t=API_TOKEN&g=1234");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -415,6 +554,7 @@ static void test_init_ping_request_rich_presence() {
   ASSERT_NUM_EQUALS(rc_api_init_ping_request(&request, &ping_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=ping&u=Username&t=API_TOKEN&g=1234&m=Level+1%2c+70%25+complete");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -432,6 +572,7 @@ static void test_init_ping_request_rich_presence_unicode() {
   ASSERT_NUM_EQUALS(rc_api_init_ping_request(&request, &ping_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=ping&u=Username&t=API_TOKEN&g=1446&m=%f0%9f%9a%b6%3a3%2c+1st+Quest");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -449,6 +590,44 @@ static void test_init_ping_request_rich_presence_empty() {
   ASSERT_NUM_EQUALS(rc_api_init_ping_request(&request, &ping_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=ping&u=Username&t=API_TOKEN&g=1234");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
+
+  rc_api_destroy_request(&request);
+}
+
+static void test_init_ping_request_game_hash_softcore() {
+  rc_api_ping_request_t ping_request;
+  rc_api_request_t request;
+
+  memset(&ping_request, 0, sizeof(ping_request));
+  ping_request.username = "Username";
+  ping_request.api_token = "API_TOKEN";
+  ping_request.game_id = 1234;
+  ping_request.game_hash = "ABCDEF0123456789";
+
+  ASSERT_NUM_EQUALS(rc_api_init_ping_request(&request, &ping_request), RC_OK);
+  ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
+  ASSERT_STR_EQUALS(request.post_data, "r=ping&u=Username&t=API_TOKEN&g=1234&h=0&x=ABCDEF0123456789");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
+
+  rc_api_destroy_request(&request);
+}
+
+static void test_init_ping_request_game_hash_hardcore() {
+  rc_api_ping_request_t ping_request;
+  rc_api_request_t request;
+
+  memset(&ping_request, 0, sizeof(ping_request));
+  ping_request.username = "Username";
+  ping_request.api_token = "API_TOKEN";
+  ping_request.game_id = 1234;
+  ping_request.hardcore = 1;
+  ping_request.game_hash = "ABCDEF0123456789";
+
+  ASSERT_NUM_EQUALS(rc_api_init_ping_request(&request, &ping_request), RC_OK);
+  ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
+  ASSERT_STR_EQUALS(request.post_data, "r=ping&u=Username&t=API_TOKEN&g=1234&h=1&x=ABCDEF0123456789");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -480,6 +659,7 @@ static void test_init_award_achievement_request_hardcore() {
   ASSERT_NUM_EQUALS(rc_api_init_award_achievement_request(&request, &award_achievement_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=awardachievement&u=Username&t=API_TOKEN&a=1234&h=1&m=ABCDEF0123456789&v=b8aefaad6f9659e2164bc60da0c3b64d");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -498,6 +678,7 @@ static void test_init_award_achievement_request_non_hardcore() {
   ASSERT_NUM_EQUALS(rc_api_init_award_achievement_request(&request, &award_achievement_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=awardachievement&u=Username&t=API_TOKEN&a=1234&h=0&m=ABABCBCBDEDEFFFF&v=ed81d6ecf825f8cbe3ae1edace098892");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -515,6 +696,7 @@ static void test_init_award_achievement_request_no_hash() {
   ASSERT_NUM_EQUALS(rc_api_init_award_achievement_request(&request, &award_achievement_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=awardachievement&u=Username&t=API_TOKEN&a=5432&h=1&v=31048257ab1788386e71ab0c222aa5c8");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -536,7 +718,7 @@ static void test_init_award_achievement_request_no_achievement_id() {
 
 static void test_process_award_achievement_response_success() {
   rc_api_award_achievement_response_t award_achievement_response;
-  const char* server_response = "{\"Success\":true,\"Score\":119102,\"AchievementID\":56481,\"AchievementsRemaining\":11}";
+  const char* server_response = "{\"Success\":true,\"Score\":119102,\"SoftcoreScore\":777,\"AchievementID\":56481,\"AchievementsRemaining\":11}";
 
   memset(&award_achievement_response, 0, sizeof(award_achievement_response));
 
@@ -544,6 +726,7 @@ static void test_process_award_achievement_response_success() {
   ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 1);
   ASSERT_PTR_NULL(award_achievement_response.response.error_message);
   ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 119102);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 777);
   ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 56481);
   ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 11);
 
@@ -552,7 +735,7 @@ static void test_process_award_achievement_response_success() {
 
 static void test_process_award_achievement_response_hardcore_already_unlocked() {
   rc_api_award_achievement_response_t award_achievement_response;
-  const char* server_response = "{\"Success\":false,\"Error\":\"User already has hardcore and regular achievements awarded.\",\"Score\":119210,\"AchievementID\":56494,\"AchievementsRemaining\":17}";
+  const char* server_response = "{\"Success\":false,\"Error\":\"User already has hardcore and regular achievements awarded.\",\"Score\":119210,\"SoftcoreScore\":777,\"AchievementID\":56494,\"AchievementsRemaining\":17}";
 
   memset(&award_achievement_response, 0, sizeof(award_achievement_response));
 
@@ -560,6 +743,7 @@ static void test_process_award_achievement_response_hardcore_already_unlocked()
   ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 1);
   ASSERT_STR_EQUALS(award_achievement_response.response.error_message, "User already has hardcore and regular achievements awarded.");
   ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 119210);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 777);
   ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 56494);
   ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 17);
 
@@ -568,7 +752,7 @@ static void test_process_award_achievement_response_hardcore_already_unlocked()
 
 static void test_process_award_achievement_response_non_hardcore_already_unlocked() {
   rc_api_award_achievement_response_t award_achievement_response;
-  const char* server_response = "{\"Success\":false,\"Error\":\"User already has this achievement awarded.\",\"Score\":119210,\"AchievementID\":56494}";
+  const char* server_response = "{\"Success\":false,\"Error\":\"User already has this achievement awarded.\",\"Score\":119210,\"SoftcoreScore\":777,\"AchievementID\":56494}";
 
   memset(&award_achievement_response, 0, sizeof(award_achievement_response));
 
@@ -576,6 +760,7 @@ static void test_process_award_achievement_response_non_hardcore_already_unlocke
   ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 1);
   ASSERT_STR_EQUALS(award_achievement_response.response.error_message, "User already has this achievement awarded.");
   ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 119210);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 777);
   ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 56494);
   ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 0xFFFFFFFF);
 
@@ -592,6 +777,7 @@ static void test_process_award_achievement_response_generic_failure() {
   ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 0);
   ASSERT_PTR_NULL(award_achievement_response.response.error_message);
   ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 0);
 
@@ -604,10 +790,11 @@ static void test_process_award_achievement_response_empty() {
 
   memset(&award_achievement_response, 0, sizeof(award_achievement_response));
 
-  ASSERT_NUM_EQUALS(rc_api_process_award_achievement_response(&award_achievement_response, server_response), RC_INVALID_JSON);
+  ASSERT_NUM_EQUALS(rc_api_process_award_achievement_response(&award_achievement_response, server_response), RC_NO_RESPONSE);
   ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 0);
   ASSERT_PTR_NULL(award_achievement_response.response.error_message);
   ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 0);
 
@@ -624,6 +811,7 @@ static void test_process_award_achievement_response_invalid_credentials() {
   ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 0);
   ASSERT_STR_EQUALS(award_achievement_response.response.error_message, "Credentials invalid (0)");
   ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 0);
 
@@ -640,6 +828,7 @@ static void test_process_award_achievement_response_text() {
   ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 0);
   ASSERT_STR_EQUALS(award_achievement_response.response.error_message, "You do not have access to that resource");
   ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 0);
 
@@ -656,6 +845,7 @@ static void test_process_award_achievement_response_no_fields() {
   ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 1);
   ASSERT_PTR_NULL(award_achievement_response.response.error_message);
   ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 0xFFFFFFFF);
 
@@ -679,6 +869,7 @@ static void test_process_award_achievement_response_429() {
   ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 0);
   ASSERT_STR_EQUALS(award_achievement_response.response.error_message, "429 Too Many Requests");
   ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 0);
   ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 0);
 
@@ -702,6 +893,81 @@ static void test_process_award_achievement_response_429_json() {
   rc_api_destroy_award_achievement_response(&award_achievement_response);
 }
 
+static void test_process_award_achievement_response_503_fancy() {
+  rc_api_award_achievement_response_t award_achievement_response;
+  const char* server_response =
+    "<!DOCTYPE html>\n"
+    "<html>\n"
+    "<head>\n"
+    "  <meta charset='utf-8'>\n"
+    "  <meta name='viewport' content='width=device-width, initial-scale=1'>\n"
+    "  <style>\n"
+    "  body {\n"
+    "    background-color: #1a1a1a;\n"
+    "    color: white; width: 100%;\n"
+    "  }\n"
+    "  a {\n"
+    "    color: #cc9900;\n"
+    "    text-decoration: none;\n"
+    "  }\n"
+    "  .center {\n"
+    "    margin: 0;\n"
+    "    padding: 0;\n"
+    "    text - align: center;\n"
+    "    position: absolute;\n"
+    "    top: 50%;\n"
+    "    left: 50%;\n"
+    "    transform: translateX(-50%) translateY(-50%);\n"
+    "  }\n"
+    "  </style>\n"
+    "  <title>503 Service Temporarily Unavailable</title>\n"
+    "</head>\n"
+    "<body style='background-color: #1a1a1a; color:white; width:100%;'>\n"
+    "<div class='center'>\n"
+    "  <img style='text-align: center; width: 75%;' src = 'https://static.retroachievements.org/assets/images/ra-logo-sm.webp'>\n"
+    "  <h1>503</h1>\n"
+    "  <h2>Service Temporarily Unavailable</h2>\n"
+    "  <img src='https://static.retroachievements.org/assets/images/cheevo/sad.webp' alt='Sad Cheevo'>\n"
+    "  <p>The RetroAchievements website is currently offline, we apologize for any inconvenience caused.</p>\n"
+    "  <p><strong><u>You can still earn achievements in any supported emulator.</u></strong></p>\n"
+    "  <p><strong>Make sure that the emulator has informed you that you have successfully logged in before you begin playing to avoid missing unlocks.</strong></p>\n"
+    "  <p>For more information and updates, please join our <a href='https://discord.gg/retroachievements'>Discord</a>.</p>\n"
+    "  <br/>\n"
+    "  <a href='https://retroachievements.org'>retroachievements.org</a>\n"
+    "</div>\n"
+    "</body>\n"
+    "</html>";
+
+  memset(&award_achievement_response, 0, sizeof(award_achievement_response));
+
+  ASSERT_NUM_EQUALS(rc_api_process_award_achievement_response(&award_achievement_response, server_response), RC_INVALID_JSON);
+  ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 0);
+  ASSERT_STR_EQUALS(award_achievement_response.response.error_message, "503 Service Temporarily Unavailable");
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 0);
+
+  rc_api_destroy_award_achievement_response(&award_achievement_response);
+}
+
+static void test_process_award_achievement_response_522_simple() {
+  rc_api_award_achievement_response_t award_achievement_response;
+  const char* server_response = "error code: 522";
+
+  memset(&award_achievement_response, 0, sizeof(award_achievement_response));
+
+  ASSERT_NUM_EQUALS(rc_api_process_award_achievement_response(&award_achievement_response, server_response), RC_INVALID_JSON);
+  ASSERT_NUM_EQUALS(award_achievement_response.response.succeeded, 0);
+  ASSERT_STR_EQUALS(award_achievement_response.response.error_message, "error code: 522");
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.new_player_score_softcore, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.awarded_achievement_id, 0);
+  ASSERT_UNUM_EQUALS(award_achievement_response.achievements_remaining, 0);
+
+  rc_api_destroy_award_achievement_response(&award_achievement_response);
+}
+
 static void test_init_submit_lboard_entry_request() {
   rc_api_submit_lboard_entry_request_t submit_lboard_entry_request;
   rc_api_request_t request;
@@ -716,6 +982,7 @@ static void test_init_submit_lboard_entry_request() {
   ASSERT_NUM_EQUALS(rc_api_init_submit_lboard_entry_request(&request, &submit_lboard_entry_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=submitlbentry&u=Username&t=API_TOKEN&i=1234&s=10999&m=ABCDEF0123456789&v=e13c9132ee651256f9d2ee8f06f75d76");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -734,6 +1001,7 @@ static void test_init_submit_lboard_entry_request_zero_value() {
   ASSERT_NUM_EQUALS(rc_api_init_submit_lboard_entry_request(&request, &submit_lboard_entry_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=submitlbentry&u=Username&t=API_TOKEN&i=1111&s=0&m=ABCDEF0123456789&v=9c2ac665157d68b8a26e83bb71dd8aaf");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -752,6 +1020,7 @@ static void test_init_submit_lboard_entry_request_negative_value() {
   ASSERT_NUM_EQUALS(rc_api_init_submit_lboard_entry_request(&request, &submit_lboard_entry_request), RC_OK);
   ASSERT_STR_EQUALS(request.url, DOREQUEST_URL);
   ASSERT_STR_EQUALS(request.post_data, "r=submitlbentry&u=Username&t=API_TOKEN&i=1111&s=-234781&m=ABCDEF0123456789&v=fbe290266f2d121a7a37942e1e90f453");
+  ASSERT_STR_EQUALS(request.content_type, RC_CONTENT_TYPE_URLENCODED);
 
   rc_api_destroy_request(&request);
 }
@@ -877,6 +1146,8 @@ void test_rapi_runtime(void) {
   TEST(test_process_fetch_game_data_response_empty);
   TEST(test_process_fetch_game_data_response_invalid_credentials);
   TEST(test_process_fetch_game_data_response_achievements);
+  TEST(test_process_fetch_game_data_response_achievement_types);
+  TEST(test_process_fetch_game_data_response_achievement_rarity);
   TEST(test_process_fetch_game_data_response_leaderboards);
   TEST(test_process_fetch_game_data_response_rich_presence);
   TEST(test_process_fetch_game_data_response_rich_presence_null);
@@ -888,6 +1159,8 @@ void test_rapi_runtime(void) {
   TEST(test_init_ping_request_rich_presence);
   TEST(test_init_ping_request_rich_presence_unicode);
   TEST(test_init_ping_request_rich_presence_empty);
+  TEST(test_init_ping_request_game_hash_softcore);
+  TEST(test_init_ping_request_game_hash_hardcore);
 
   TEST(test_process_ping_response);
 
@@ -907,6 +1180,8 @@ void test_rapi_runtime(void) {
   TEST(test_process_award_achievement_response_no_fields);
   TEST(test_process_award_achievement_response_429);
   TEST(test_process_award_achievement_response_429_json);
+  TEST(test_process_award_achievement_response_503_fancy);
+  TEST(test_process_award_achievement_response_522_simple);
 
   /* submitlbentry */
   TEST(test_init_submit_lboard_entry_request);
