@@ -3,7 +3,7 @@
 #include "../test_framework.h"
 #include "mock_memory.h"
 
-static void _assert_operand(rc_operand_t* self, char expected_type, char expected_size, unsigned expected_address) {
+static void _assert_operand(rc_operand_t* self, uint8_t expected_type, uint8_t expected_size, uint32_t expected_address) {
   ASSERT_NUM_EQUALS(self->type, expected_type);
 
   switch (expected_type) {
@@ -22,11 +22,11 @@ static void _assert_operand(rc_operand_t* self, char expected_type, char expecte
 #define assert_operand(operand, expected_type, expected_size, expected_address) ASSERT_HELPER(_assert_operand(operand, expected_type, expected_size, expected_address), "assert_operand")
 
 static void _assert_parse_condition(
-    const char* memaddr, char expected_type,
-    char expected_left_type, char expected_left_size, unsigned expected_left_value,
-    char expected_operator,
-    char expected_right_type, char expected_right_size, unsigned expected_right_value,
-    unsigned expected_required_hits
+    const char* memaddr, uint8_t expected_type,
+    uint8_t expected_left_type, uint8_t expected_left_size, uint32_t expected_left_value,
+    uint8_t expected_operator,
+    uint8_t expected_right_type, uint8_t expected_right_size, uint32_t expected_right_value,
+    uint32_t expected_required_hits
 ) {
     rc_condition_t* self;
     rc_parse_state_t parse;
@@ -49,8 +49,8 @@ static void _assert_parse_condition(
     ASSERT_HELPER(_assert_parse_condition(memaddr, expected_type, expected_left_type, expected_left_size, expected_left_value, \
                                           expected_operator, expected_right_type, expected_right_size, expected_right_value, expected_required_hits), "assert_parse_condition")
 
-static void test_parse_condition(const char* memaddr, char expected_type, char expected_left_type,
-    char expected_operator, int expected_required_hits) {
+static void test_parse_condition(const char* memaddr, uint8_t expected_type, uint8_t expected_left_type,
+    uint8_t expected_operator, uint32_t expected_required_hits) {
   if (expected_operator == RC_OPERATOR_NONE) {
     assert_parse_condition(memaddr, expected_type,
       expected_left_type, RC_MEMSIZE_8_BITS, 0x1234U,
@@ -70,8 +70,8 @@ static void test_parse_condition(const char* memaddr, char expected_type, char e
 }
 
 static void test_parse_operands(const char* memaddr,
-    char expected_left_type, char expected_left_size, unsigned expected_left_value,
-    char expected_right_type, char expected_right_size, unsigned expected_right_value) {
+    uint8_t expected_left_type, uint8_t expected_left_size, uint32_t expected_left_value,
+    uint8_t expected_right_type, uint8_t expected_right_size, uint32_t expected_right_value) {
   assert_parse_condition(memaddr, RC_CONDITION_STANDARD,
     expected_left_type, expected_left_size, expected_left_value,
     RC_OPERATOR_EQ,
@@ -80,7 +80,7 @@ static void test_parse_operands(const char* memaddr,
   );
 }
 
-static void test_parse_modifier(const char* memaddr, char expected_operator, char expected_operand, double expected_multiplier) {
+static void test_parse_modifier(const char* memaddr, uint8_t expected_operator, uint8_t expected_operand, double expected_multiplier) {
   assert_parse_condition(memaddr, RC_CONDITION_ADD_SOURCE,
     RC_OPERAND_ADDRESS, RC_MEMSIZE_8_BITS, 0x1234U,
     expected_operator,
@@ -89,7 +89,7 @@ static void test_parse_modifier(const char* memaddr, char expected_operator, cha
   );
 }
 
-static void test_parse_modifier_shorthand(const char* memaddr, char expected_type) {
+static void test_parse_modifier_shorthand(const char* memaddr, uint8_t expected_type) {
   assert_parse_condition(memaddr, expected_type,
     RC_OPERAND_ADDRESS, RC_MEMSIZE_8_BITS, 0x1234U,
     RC_OPERATOR_NONE,
@@ -117,13 +117,13 @@ static int evaluate_condition(rc_condition_t* cond, memory_t* memory, rc_memref_
   return rc_test_condition(cond, &eval_state);
 }
 
-static void test_evaluate_condition(const char* memaddr, int expected_comparator, int expected_result) {
+static void test_evaluate_condition(const char* memaddr, uint8_t expected_comparator, int expected_result) {
   rc_condition_t* self;
   rc_parse_state_t parse;
   char buffer[512];
   rc_memref_t* memrefs;
   int ret;
-  unsigned char ram[] = {0x00, 0x11, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x11, 0x34, 0xAB, 0x56};
   memory_t memory;
 
   memory.ram = ram;
@@ -179,7 +179,7 @@ static void test_evaluate_condition_float(const char* memaddr, int expected_resu
   char buffer[512];
   rc_memref_t* memrefs;
   int ret;
-  unsigned char ram[] = {0x00, 0x00, 0x00, 0x40, 0x83, 0x49, 0x0F, 0xDB}; /* FF0=2, FF4=2*pi */
+  uint8_t ram[] = {0x00, 0x00, 0x00, 0x40, 0x83, 0x49, 0x0F, 0xDB}; /* FF0=2, FF4=2*pi */
   memory_t memory;
 
   memory.ram = ram;
@@ -203,7 +203,7 @@ static void test_evaluate_condition_float(const char* memaddr, int expected_resu
 }
 
 static void test_condition_compare_delta() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_condition_t* cond;
   rc_parse_state_t parse;
@@ -237,7 +237,7 @@ static void test_condition_compare_delta() {
 }
 
 static void test_condition_delta_24bit() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_condition_t* cond;
   rc_parse_state_t parse;
@@ -283,7 +283,7 @@ static void test_condition_delta_24bit() {
 }
 
 static void test_condition_prior_24bit() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_condition_t* cond;
   rc_parse_state_t parse;
