@@ -2,17 +2,17 @@
 
 #include "../test_framework.h"
 
-static void test_name(int console_id, const char* expected_name)
+static void test_name(uint32_t console_id, const char* expected_name)
 {
   ASSERT_STR_EQUALS(rc_console_name(console_id), expected_name);
 }
 
-static void test_memory(int console_id, unsigned expected_total_memory)
+static void test_memory(uint32_t console_id, uint32_t expected_total_memory)
 {
   const rc_memory_regions_t* regions = rc_console_memory_regions(console_id);
-  unsigned total_memory = 0;
-  unsigned max_address = 0;
-  unsigned i;
+  uint32_t total_memory = 0;
+  uint32_t max_address = 0;
+  uint32_t i;
   ASSERT_PTR_NOT_NULL(regions);
 
   if (expected_total_memory == 0)
@@ -123,8 +123,10 @@ void test_consoleinfo(void) {
 
   TEST_PARAMS2(test_name, 100, "Hubs");
   TEST_PARAMS2(test_name, 101, "Events");
+  TEST_PARAMS2(test_name, 102, "Standalone");
 
   /* memory maps */
+  TEST_PARAMS2(test_memory, RC_CONSOLE_UNKNOWN, 0x000000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_3DO, 0x200000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_AMIGA, 0x100000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_AMSTRAD_PC, 0x090000);
@@ -144,7 +146,7 @@ void test_consoleinfo(void) {
   TEST_PARAMS2(test_memory, RC_CONSOLE_FAIRCHILD_CHANNEL_F, 0x010C40);
   TEST_PARAMS2(test_memory, RC_CONSOLE_GAMEBOY, 0x010000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_GAMEBOY_COLOR, 0x016000);
-  TEST_PARAMS2(test_memory, RC_CONSOLE_GAMEBOY_ADVANCE, 0x048000);
+  TEST_PARAMS2(test_memory, RC_CONSOLE_GAMEBOY_ADVANCE, 0x058000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_GAMECUBE, 0x01800000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_GAME_GEAR, 0x002000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_INTELLIVISION, 0x040080);
@@ -154,6 +156,7 @@ void test_consoleinfo(void) {
   TEST_PARAMS2(test_memory, RC_CONSOLE_MEGA_DRIVE, 0x020000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_MEGADUCK, 0x010000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_MSX, 0x080000);
+  TEST_PARAMS2(test_memory, RC_CONSOLE_MS_DOS, 0x4200000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_NEOGEO_POCKET, 0x004000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_NEO_GEO_CD, 0x010000);
   TEST_PARAMS2(test_memory, RC_CONSOLE_NINTENDO, 0x010000);
