@@ -1,14 +1,14 @@
 #include "rc_validate.h"
 
-#include "rc_compat.h"
 #include "rc_consoles.h"
 
+#include "../rc_compat.h"
 #include "../test_framework.h"
 
 #include <stdlib.h>
 #include <string.h>
 
-int validate_trigger(const char* trigger, char result[], const size_t result_size, unsigned max_address) {
+int validate_trigger(const char* trigger, char result[], const size_t result_size, uint32_t max_address) {
   char* buffer;
   rc_trigger_t* compiled;
   int success = 0;
@@ -25,7 +25,7 @@ int validate_trigger(const char* trigger, char result[], const size_t result_siz
   if (compiled == NULL) {
     snprintf(result, result_size, "parse failed");
   }
-  else if (*(unsigned*)&buffer[ret] != 0xCDCDCDCD) {
+  else if (*(uint32_t*)&buffer[ret] != 0xCDCDCDCD) {
     snprintf(result, result_size, "write past end of buffer");
   }
   else if (rc_validate_trigger(compiled, result, result_size, max_address)) {
@@ -36,7 +36,7 @@ int validate_trigger(const char* trigger, char result[], const size_t result_siz
   return success;
 }
 
-static void test_validate_trigger_max_address(const char* trigger, const char* expected_error, unsigned max_address) {
+static void test_validate_trigger_max_address(const char* trigger, const char* expected_error, uint32_t max_address) {
   char buffer[512];
   int valid = validate_trigger(trigger, buffer, sizeof(buffer), max_address);
 
@@ -62,7 +62,7 @@ static void test_validate_trigger_128k(const char* trigger, const char* expected
   test_validate_trigger_max_address(trigger, expected_error, 0x1FFFF);
 }
 
-int validate_trigger_for_console(const char* trigger, char result[], const size_t result_size, int console_id) {
+int validate_trigger_for_console(const char* trigger, char result[], const size_t result_size, uint32_t console_id) {
   char* buffer;
   rc_trigger_t* compiled;
   int success = 0;
@@ -90,7 +90,7 @@ int validate_trigger_for_console(const char* trigger, char result[], const size_
   return success;
 }
 
-static void test_validate_trigger_console(const char* trigger, const char* expected_error, int console_id) {
+static void test_validate_trigger_console(const char* trigger, const char* expected_error, uint32_t console_id) {
   char buffer[512];
   int valid = validate_trigger_for_console(trigger, buffer, sizeof(buffer), console_id);
 
@@ -267,11 +267,11 @@ void test_float_comparisons() {
   TEST_PARAMS2(test_validate_trigger, "fF1234=2", "");
   TEST_PARAMS2(test_validate_trigger, "0xX1234=2", "");
   TEST_PARAMS2(test_validate_trigger, "0xX1234=f2.3", "Condition 1: Comparison is never true"); /* non integral comparison */
-  TEST_PARAMS2(test_validate_trigger, "0xX1234!=f2.3", "Condition 1: Comparison is never true"); /* non integral comparison */
-  TEST_PARAMS2(test_validate_trigger, "0xX1234<f2.3", "Condition 1: Comparison is never true"); /* non integral comparison */
-  TEST_PARAMS2(test_validate_trigger, "0xX1234<=f2.3", "Condition 1: Comparison is never true"); /* non integral comparison */
-  TEST_PARAMS2(test_validate_trigger, "0xX1234>f2.3", "Condition 1: Comparison is never true"); /* non integral comparison */
-  TEST_PARAMS2(test_validate_trigger, "0xX1234>=f2.3", "Condition 1: Comparison is never true"); /* non integral comparison */
+  TEST_PARAMS2(test_validate_trigger, "0xX1234!=f2.3", "Condition 1: Comparison is always true"); /* non integral comparison */
+  TEST_PARAMS2(test_validate_trigger, "0xX1234<f2.3", ""); /* will be converted to < 3 */
+  TEST_PARAMS2(test_validate_trigger, "0xX1234<=f2.3", ""); /* will be converted to <= 2 */
+  TEST_PARAMS2(test_validate_trigger, "0xX1234>f2.3", ""); /* will be converted to > 2 */
+  TEST_PARAMS2(test_validate_trigger, "0xX1234>=f2.3", ""); /* will be converted to >= 3 */
   TEST_PARAMS2(test_validate_trigger, "0xX1234=f2.0", ""); /* float can be converted to int without loss of data*/
   TEST_PARAMS2(test_validate_trigger, "0xH1234=f2.3", "Condition 1: Comparison is never true");
   TEST_PARAMS2(test_validate_trigger, "0xH1234=f300.0", "Condition 1: Comparison is never true"); /* value out of range */
@@ -285,6 +285,16 @@ void test_float_comparisons() {
   TEST_PARAMS2(test_validate_trigger, "fM1234>f-2.3", "");
   TEST_PARAMS2(test_validate_trigger, "I:0xX2345_fM1234>f1.0", "");
   TEST_PARAMS2(test_validate_trigger, "I:0xX2345_fM1234>f-1.0", "");
+  TEST_PARAMS2(test_validate_trigger, "fF1234>=f0.0", ""); /* explicit float comparison can be negative */
+  TEST_PARAMS2(test_validate_trigger, "fM1234>=f0.0", "");
+  TEST_PARAMS2(test_validate_trigger, "fB1234>=f0.0", "");
+  TEST_PARAMS2(test_validate_trigger, "fF1234>=0", ""); /* implicit float comparison can be negative */
+  TEST_PARAMS2(test_validate_trigger, "fM1234>=0", "");
+  TEST_PARAMS2(test_validate_trigger, "fB1234>=0", "");
+  TEST_PARAMS2(test_validate_trigger, "0xH1234>=f0.1", ""); /* 0 can be less than 0.1 */
+  TEST_PARAMS2(test_validate_trigger, "0xH1234>=f255.1", "Condition 1: Comparison is never true"); /* 255 cannot be >= 255.1 */
+  TEST_PARAMS2(test_validate_trigger, "f0.1<=0xH1234", ""); /* 0 can be less than 0.1 */
+  TEST_PARAMS2(test_validate_trigger, "f255.1<=0xH1234", "Condition 1: Comparison is never true"); /* 255 cannot be >= 255.1 */
 }
 
 void test_conflicting_conditions() {
@@ -348,6 +358,12 @@ void test_redundant_conditions() {
   TEST_PARAMS2(test_validate_trigger, "0xH0000!=0S0xH0000!=0S0xH0001=2", "Alt1 Condition 1: Redundant with Core Condition 1");
 }
 
+void test_redundant_hitcounts() {
+  TEST_PARAMS2(test_validate_trigger, "R:0xH0000!=0", "");
+  TEST_PARAMS2(test_validate_trigger, "R:0xH0000!=0.1.", "Condition 1: Hit target of 1 is redundant on ResetIf");
+  TEST_PARAMS2(test_validate_trigger, "R:0xH0000!=0.2.", "");
+}
+
 void test_rc_validate(void) {
   TEST_SUITE_BEGIN();
 
@@ -364,6 +380,7 @@ void test_rc_validate(void) {
   test_float_comparisons();
   test_conflicting_conditions();
   test_redundant_conditions();
+  test_redundant_hitcounts();
 
   TEST_SUITE_END();
 }
