@@ -58,7 +58,7 @@ static rc_condition_t* trigger_get_cond(rc_trigger_t* trigger, int group_index,
   return cond;
 }
 
-static void _assert_hit_count(rc_trigger_t* trigger, int group_index, int cond_index, unsigned expected_hit_count) {
+static void _assert_hit_count(rc_trigger_t* trigger, int group_index, int cond_index, uint32_t expected_hit_count) {
   rc_condition_t* cond = trigger_get_cond(trigger, group_index, cond_index);
   ASSERT_PTR_NOT_NULL(cond);
 
@@ -73,7 +73,7 @@ static int evaluate_trigger(rc_trigger_t* self, memory_t* memory) {
 /* ======================================================== */
 
 static void test_alt_groups() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -119,7 +119,7 @@ static void test_alt_groups() {
 }
 
 static void test_empty_core() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -154,7 +154,7 @@ static void test_empty_core() {
 }
 
 static void test_empty_alt() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -183,7 +183,7 @@ static void test_empty_alt() {
 }
 
 static void test_empty_last_alt() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -212,7 +212,7 @@ static void test_empty_last_alt() {
 }
 
 static void test_empty_all_alts() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -233,7 +233,7 @@ static void test_empty_all_alts() {
 }
 
 static void test_resetif_in_alt_group() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[1024];
@@ -272,7 +272,7 @@ static void test_resetif_in_alt_group() {
 }
 
 static void test_pauseif_in_alt_group() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[1024];
@@ -311,7 +311,7 @@ static void test_pauseif_in_alt_group() {
 }
 
 static void test_pauseif_resetif_in_alt_group() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[1024];
@@ -358,7 +358,7 @@ static void test_pauseif_resetif_in_alt_group() {
 }
 
 static void test_pauseif_hitcount_with_reset() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -409,7 +409,7 @@ static void test_pauseif_hitcount_with_reset() {
 }
 
 static void test_measured() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[256];
@@ -447,7 +447,7 @@ static void test_measured() {
 }
 
 static void test_measured_as_percent() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[256];
@@ -485,7 +485,7 @@ static void test_measured_as_percent() {
 }
 
 static void test_measured_comparison() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[256];
@@ -526,7 +526,7 @@ static void test_measured_comparison() {
 }
 
 static void test_measured_addhits() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[256];
@@ -586,7 +586,7 @@ static void test_measured_addhits() {
 }
 
 static void test_measured_indirect() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[256];
@@ -626,7 +626,7 @@ static void test_measured_indirect() {
 }
 
 static void test_measured_multiple() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -696,7 +696,7 @@ static void test_measured_multiple() {
 }
 
 static void test_measured_multiple_with_hitcount_in_core() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -731,7 +731,7 @@ static void test_measured_multiple_with_hitcount_in_core() {
 }
 
 static void test_measured_while_paused() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -763,7 +763,7 @@ static void test_measured_while_paused() {
 }
 
 static void test_measured_while_paused_multiple() {
-  unsigned char ram[] = {0x00, 0x00, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x00, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -830,7 +830,7 @@ static void test_measured_while_paused_multiple() {
 }
 
 static void test_measured_while_paused_reset_alt() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -872,7 +872,7 @@ static void test_measured_while_paused_reset_alt() {
 }
 
 static void test_measured_while_paused_reset_core() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -914,7 +914,7 @@ static void test_measured_while_paused_reset_core() {
 }
 
 static void test_measured_while_paused_reset_non_hitcount() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -948,7 +948,7 @@ static void test_measured_while_paused_reset_non_hitcount() {
 }
 
 static void test_measured_reset_hitcount() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -995,7 +995,7 @@ static void test_measured_reset_hitcount() {
 }
 
 static void test_measured_reset_comparison() {
-  unsigned char ram[] = {0x00, 0x12, 0x02, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x02, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1045,7 +1045,7 @@ static void test_measured_reset_comparison() {
 }
 
 static void test_measured_if() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[256];
@@ -1085,7 +1085,7 @@ static void test_measured_if() {
 }
 
 static void test_measured_if_comparison() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[256];
@@ -1133,7 +1133,7 @@ static void test_measured_if_comparison() {
 }
 
 static void test_measured_if_multiple_measured() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1213,7 +1213,7 @@ static void test_measured_if_multiple_measured() {
 }
 
 static void test_measured_if_multiple_measured_if() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1272,7 +1272,7 @@ static void test_measured_if_multiple_measured_if() {
 }
 
 static void test_measured_if_while_paused() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1321,8 +1321,54 @@ static void test_measured_if_while_paused() {
   ASSERT_NUM_EQUALS(trigger->measured_value, 2U);
 }
 
+static void test_measured_trigger() {
+  uint8_t ram[] = { 0x00, 0x12, 0x34, 0xAB, 0x56 };
+  memory_t memory;
+  rc_trigger_t* trigger;
+  char buffer[512];
+
+  memory.ram = ram;
+  memory.size = sizeof(ram);
+
+  /* never(byte(0) != 0) && trigger_when(measured(repeated(3, byte(2) == 52))) */
+  assert_parse_trigger(&trigger, buffer, "R:0xH0000!=0SM:0xH0002=52(3)ST:0=1");
+  ASSERT_NUM_EQUALS(trigger->measured_as_percent, 0);
+
+  /* condition is true - hit count should be incremented, and trigger shown */
+  ASSERT_NUM_EQUALS(evaluate_trigger(trigger, &memory), RC_TRIGGER_STATE_PRIMED);
+  assert_hit_count(trigger, 1, 0, 1U);
+  ASSERT_NUM_EQUALS(trigger->measured_value, 1U);
+  ASSERT_NUM_EQUALS(trigger->measured_target, 3U);
+
+  /* core condition is false - trigger should not be shown and hit count reset */
+  ram[0] = 1;
+  ASSERT_NUM_EQUALS(evaluate_trigger(trigger, &memory), RC_TRIGGER_STATE_RESET);
+  assert_hit_count(trigger, 1, 0, 0U);
+  ASSERT_NUM_EQUALS(trigger->measured_value, 0U);
+  ASSERT_NUM_EQUALS(trigger->measured_target, 3U);
+
+  /* core condition is true again - hit count should be incremented, and trigger shown */
+  ram[0] = 0;
+  ASSERT_NUM_EQUALS(evaluate_trigger(trigger, &memory), RC_TRIGGER_STATE_PRIMED);
+  assert_hit_count(trigger, 1, 0, 1U);
+  ASSERT_NUM_EQUALS(trigger->measured_value, 1U);
+  ASSERT_NUM_EQUALS(trigger->measured_target, 3U);
+
+  /* increment hit count */
+  ASSERT_NUM_EQUALS(evaluate_trigger(trigger, &memory), RC_TRIGGER_STATE_PRIMED);
+  assert_hit_count(trigger, 1, 0, 2U);
+  ASSERT_NUM_EQUALS(trigger->measured_value, 2U);
+  ASSERT_NUM_EQUALS(trigger->measured_target, 3U);
+
+  /* trigger */
+  ASSERT_NUM_EQUALS(evaluate_trigger(trigger, &memory), RC_TRIGGER_STATE_TRIGGERED);
+  assert_hit_count(trigger, 1, 0, 3U);
+  ASSERT_NUM_EQUALS(trigger->measured_value, 3U);
+  ASSERT_NUM_EQUALS(trigger->measured_target, 3U);
+}
+
 static void test_resetnextif_trigger() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1382,7 +1428,7 @@ static void test_resetnextif_trigger() {
 }
 
 static void test_evaluate_trigger_inactive() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1420,7 +1466,7 @@ static void test_evaluate_trigger_inactive() {
 }
 
 static void test_evaluate_trigger_waiting() {
-  unsigned char ram[] = {0x00, 0x12, 0x18, 0xAB, 0x09};
+  uint8_t ram[] = {0x00, 0x12, 0x18, 0xAB, 0x09};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1460,7 +1506,7 @@ static void test_evaluate_trigger_waiting() {
 }
 
 static void test_evaluate_trigger_reset() {
-  unsigned char ram[] = {0x00, 0x05, 0x10, 0xAB, 0x09};
+  uint8_t ram[] = {0x00, 0x05, 0x10, 0xAB, 0x09};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1488,7 +1534,7 @@ static void test_evaluate_trigger_reset() {
 }
 
 static void test_evaluate_trigger_reset_next() {
-  unsigned char ram[] = {0x00, 0x05, 0x10, 0xAB, 0x09};
+  uint8_t ram[] = {0x00, 0x05, 0x10, 0xAB, 0x09};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1543,7 +1589,7 @@ static void test_evaluate_trigger_reset_next() {
 }
 
 static void test_evaluate_trigger_triggered() {
-  unsigned char ram[] = {0x00, 0x05, 0x10, 0xAB, 0x09};
+  uint8_t ram[] = {0x00, 0x05, 0x10, 0xAB, 0x09};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1579,7 +1625,7 @@ static void test_evaluate_trigger_triggered() {
 }
 
 static void test_evaluate_trigger_paused() {
-  unsigned char ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
+  uint8_t ram[] = {0x00, 0x12, 0x34, 0xAB, 0x56};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1647,7 +1693,7 @@ static void test_evaluate_trigger_paused() {
 }
 
 static void test_evaluate_trigger_primed() {
-  unsigned char ram[] = {0x00, 0x01, 0x00, 0x01, 0x00};
+  uint8_t ram[] = {0x00, 0x01, 0x00, 0x01, 0x00};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1688,7 +1734,7 @@ static void test_evaluate_trigger_primed() {
 }
 
 static void test_evaluate_trigger_primed_in_alts() {
-  unsigned char ram[] = {0x01, 0x00, 0x00, 0x00, 0x00};
+  uint8_t ram[] = {0x01, 0x00, 0x00, 0x00, 0x00};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[1024];
@@ -1724,7 +1770,7 @@ static void test_evaluate_trigger_primed_in_alts() {
 }
 
 static void test_evaluate_trigger_primed_one_alt() {
-  unsigned char ram[] = {0x00, 0x00, 0x00, 0x00, 0x00};
+  uint8_t ram[] = {0x00, 0x00, 0x00, 0x00, 0x00};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1756,7 +1802,7 @@ static void test_evaluate_trigger_primed_one_alt() {
 }
 
 static void test_evaluate_trigger_disabled() {
-  unsigned char ram[] = {0x00, 0x00, 0x00, 0x00, 0x00};
+  uint8_t ram[] = {0x00, 0x00, 0x00, 0x00, 0x00};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1773,7 +1819,7 @@ static void test_evaluate_trigger_disabled() {
 }
 
 static void test_evaluate_trigger_chained_resetnextif() {
-  unsigned char ram[] = {0x00, 0x00, 0x00, 0x00, 0x00};
+  uint8_t ram[] = {0x00, 0x00, 0x00, 0x00, 0x00};
   memory_t memory;
   rc_trigger_t* trigger;
   char buffer[512];
@@ -1934,6 +1980,7 @@ void test_trigger(void) {
   TEST(test_measured_if_multiple_measured);
   TEST(test_measured_if_multiple_measured_if);
   TEST(test_measured_if_while_paused);
+  TEST(test_measured_trigger);
 
   /* trigger */
   TEST(test_resetnextif_trigger);
