@@ -1,13 +1,13 @@
 #include "rc_hash.h"
 
-#include "../rcheevos/rc_compat.h"
+#include "../rc_compat.h"
 #include "../test_framework.h"
 #include "data.h"
 #include "mock_filereader.h"
 
 #include <stdlib.h>
 
-static int hash_mock_file(const char* filename, char hash[33], int console_id, const uint8_t* buffer, size_t buffer_size)
+static int hash_mock_file(const char* filename, char hash[33], uint32_t console_id, const uint8_t* buffer, size_t buffer_size)
 {
   mock_file(0, filename, buffer, buffer_size);
 
@@ -23,7 +23,7 @@ static void iterate_mock_file(struct rc_hash_iterator *iterator, const char* fil
 
 /* ========================================================================= */
 
-static void test_hash_full_file(int console_id, const char* filename, size_t size, const char* expected_md5)
+static void test_hash_full_file(uint32_t console_id, const char* filename, size_t size, const char* expected_md5)
 {
   uint8_t* image = generate_generic_file(size);
   char hash_buffer[33], hash_file[33], hash_iterator[33];
@@ -56,7 +56,7 @@ static void test_hash_full_file(int console_id, const char* filename, size_t siz
   ASSERT_STR_EQUALS(hash_iterator, expected_md5);
 }
 
-static void test_hash_unknown_format(int console_id, const char* path)
+static void test_hash_unknown_format(uint32_t console_id, const char* path)
 {
   char hash_file[33] = "", hash_iterator[33] = "";
 
@@ -79,7 +79,7 @@ static void test_hash_unknown_format(int console_id, const char* path)
   ASSERT_STR_EQUALS(hash_iterator, "");
 }
 
-static void test_hash_m3u(int console_id, const char* filename, size_t size, const char* expected_md5)
+static void test_hash_m3u(uint32_t console_id, const char* filename, size_t size, const char* expected_md5)
 {
   uint8_t* image = generate_generic_file(size);
   char hash_file[33], hash_iterator[33];
@@ -110,7 +110,7 @@ static void test_hash_m3u(int console_id, const char* filename, size_t size, con
   ASSERT_STR_EQUALS(hash_iterator, expected_md5);
 }
 
-static void test_hash_filename(int console_id, const char* path, const char* expected_md5)
+static void test_hash_filename(uint32_t console_id, const char* path, const char* expected_md5)
 {
   char hash_file[33], hash_iterator[33];
 
@@ -886,6 +886,79 @@ static void test_hash_gamecube()
 
 /* ========================================================================= */
 
+static void test_hash_msdos_dosz()
+{
+  size_t image_size;
+  uint8_t* image = generate_zip_file(&image_size);
+  char hash_file[33], hash_iterator[33];
+  const char* expected_md5 = "59a255662262f5ada32791b8c36e8ea7";
+
+  mock_file(0, "game.dosz", image, image_size);
+
+  /* test file hash */
+  int result_file = rc_hash_generate_from_file(hash_file, RC_CONSOLE_MS_DOS, "game.dosz");
+
+  /* test file identification from iterator */
+  struct rc_hash_iterator iterator;
+  rc_hash_initialize_iterator(&iterator, "game.dosz", NULL, 0);
+  int result_iterator = rc_hash_iterate(hash_iterator, &iterator);
+  rc_hash_destroy_iterator(&iterator);
+
+  /* cleanup */
+  free(image);
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_file, 1);
+  ASSERT_STR_EQUALS(hash_file, expected_md5);
+
+  ASSERT_NUM_EQUALS(result_iterator, 1);
+  ASSERT_STR_EQUALS(hash_iterator, expected_md5);
+}
+
+static void test_hash_msdos_dosz_zip64()
+{
+  size_t image_size;
+  uint8_t* image = generate_zip64_file(&image_size);
+  char hash_file[33];
+  const char* expected_md5 = "927dad0a57a2860267ab7bcdb8bc3f61";
+
+  mock_file(0, "game.dosz", image, image_size);
+
+  /* test file hash */
+  int result_file = rc_hash_generate_from_file(hash_file, RC_CONSOLE_MS_DOS, "game.dosz");
+
+  /* cleanup */
+  free(image);
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_file, 1);
+  ASSERT_STR_EQUALS(hash_file, expected_md5);
+}
+
+static void test_hash_msdos_dosz_with_dosc()
+{
+  size_t image_size;
+  uint8_t* image = generate_zip_file(&image_size);
+  char hash_dosc[33];
+  const char* expected_dosc_md5 = "dd0c0b0c170c30722784e5e962764c35";
+
+  /* Add main dosz file and overlay dosc file which will get hashed together */
+  mock_file(0, "game.dosz", image, image_size);
+  mock_file(1, "game.dosc", image, image_size);
+
+  /* test file hash */
+  int result_dosc = rc_hash_generate_from_file(hash_dosc, RC_CONSOLE_MS_DOS, "game.dosz");
+
+  /* cleanup */
+  free(image);
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_dosc, 1);
+  ASSERT_STR_EQUALS(hash_dosc, expected_dosc_md5);
+}
+
+/* ========================================================================= */
+
 static void test_hash_nes_32k()
 {
   size_t image_size;
@@ -1514,8 +1587,8 @@ static void test_hash_psx_cd_no_system_cnf()
   uint8_t* image;
   char hash_file[33], hash_iterator[33];
   const char* expected_md5 = "e494c79a7315be0dc3e8571c45df162c";
-  unsigned binary_size = 0x12000;
-  const unsigned sectors_needed = (((binary_size + 2047) / 2048) + 20);
+  uint32_t binary_size = 0x12000;
+  const uint32_t sectors_needed = (((binary_size + 2047) / 2048) + 20);
   uint8_t* exe;
 
   image = generate_iso9660_bin(sectors_needed, "HOMEBREW", &image_size);
@@ -1690,7 +1763,7 @@ static void test_hash_psp()
   size_t image_size;
   uint8_t* image = generate_iso9660_bin(160, "TEST", &image_size);
   char hash_file[33], hash_iterator[33];
-  const char* expected_md5 = "80c0b42b2d89d036086869433a176a03";
+  const char* expected_md5 = "27ec2f9b7238b2ef29af31ddd254f201";
 
   generate_iso9660_file(image, "PSP_GAME\\PARAM.SFO", param_sfo, param_sfo_size);
   generate_iso9660_file(image, "PSP_GAME\\SYSDIR\\EBOOT.BIN", eboot_bin, eboot_bin_size);
@@ -1855,6 +1928,38 @@ static void test_hash_saturn_invalid_header()
   free(image);
 }
 
+static void test_hash_scv_cart()
+{
+  size_t image_size = 32768 + 32;
+  uint8_t* image = generate_generic_file(image_size);
+  char hash_file[33], hash_iterator[33];
+  const char* expected_md5 = "4309c9844b44f9ff8256dfc04687b8fd";
+
+  memcpy(image, "EmuSCV....CART..................", 32);
+
+  mock_file(0, "game.cart", image, image_size);
+  /* test file hash */
+  int result_file = rc_hash_generate_from_file(hash_file, RC_CONSOLE_SUPER_CASSETTEVISION, "game.cart");
+
+  /* test file identification from iterator */
+  int result_iterator;
+  struct rc_hash_iterator iterator;
+
+  rc_hash_initialize_iterator(&iterator, "game.cart", NULL, 0);
+  result_iterator = rc_hash_iterate(hash_iterator, &iterator);
+  rc_hash_destroy_iterator(&iterator);
+
+  /* cleanup */
+  free(image);
+
+  /* validation */
+  ASSERT_NUM_EQUALS(result_file, 1);
+  ASSERT_STR_EQUALS(hash_file, expected_md5);
+
+  ASSERT_NUM_EQUALS(result_iterator, 1);
+  ASSERT_STR_EQUALS(hash_iterator, expected_md5);
+}
+
 /* ========================================================================= */
 
 static void assert_valid_m3u(const char* disc_filename, const char* m3u_filename, const char* m3u_contents)
@@ -2157,6 +2262,11 @@ void test_hash(void) {
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_MSX, "test.dsk", 737280, "0e73fe94e5f2e2d8216926eae512b7a6");
   TEST_PARAMS4(test_hash_m3u, RC_CONSOLE_MSX, "test.dsk", 737280, "0e73fe94e5f2e2d8216926eae512b7a6");
 
+  /* MS DOS */
+  TEST(test_hash_msdos_dosz);
+  TEST(test_hash_msdos_dosz_zip64);
+  TEST(test_hash_msdos_dosz_with_dosc);
+
   /* Neo Geo CD */
   TEST(test_hash_neogeocd);
   TEST(test_hash_neogeocd_multiple_prg);
@@ -2251,6 +2361,10 @@ void test_hash(void) {
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_SUPER_NINTENDO, "test.smc", 524288, "68f0f13b598e0b66461bc578375c3888");
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_SUPER_NINTENDO, "test.smc", 524288 + 512, "258c93ebaca1c3f488ab48218e5e8d38");
 
+  /* Super Cassette Vision */
+  TEST(test_hash_scv_cart);
+  TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_SUPER_CASSETTEVISION, "test.bin", 32768, "6a2305a2b6675a97ff792709be1ca857");
+
   /* TI-83 */
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_TI83, "test.83g", 1695, "bfb6048395a425c69743900785987c42");
   TEST_PARAMS4(test_hash_full_file, RC_CONSOLE_TI83, "test.83p", 2500, "6e81d530ee9a79d4f4f505729ad74bb5");
