@@ -170,7 +170,7 @@ static void append_invalid_trigger_condition(char result[], size_t result_size,
   }
 }
 
-static int validate_trigger(const char* trigger, char result[], size_t result_size, int console_id) {
+static int validate_trigger(const char* trigger, char result[], size_t result_size, uint32_t console_id) {
   char* buffer;
   rc_trigger_t* compiled;
   int success = 0;
@@ -207,7 +207,7 @@ static int validate_trigger(const char* trigger, char result[], size_t result_si
   return success;
 }
 
-static int validate_leaderboard(const char* leaderboard, char result[], const size_t result_size, int console_id)
+static int validate_leaderboard(const char* leaderboard, char result[], const size_t result_size, uint32_t console_id)
 {
   char* buffer;
   rc_lboard_t* compiled;
@@ -296,7 +296,7 @@ static int validate_leaderboard(const char* leaderboard, char result[], const si
 
 static int validate_macros(const rc_richpresence_t* richpresence, const char* script, char result[], const size_t result_size)
 {
-  const unsigned short RC_FORMAT_UNKNOWN_MACRO = 103; /* enum not exposed by header */
+  const uint16_t RC_FORMAT_UNKNOWN_MACRO = 103; /* enum not exposed by header */
 
   rc_richpresence_display_t* display = richpresence->first_display;
   while (display != NULL) {
@@ -349,7 +349,7 @@ static int validate_macros(const rc_richpresence_t* richpresence, const char* sc
   return 1;
 }
 
-static int validate_richpresence(const char* script, char result[], const size_t result_size, int console_id)
+static int validate_richpresence(const char* script, char result[], const size_t result_size, uint32_t console_id)
 {
   char* buffer;
   rc_richpresence_t* compiled;
